package com.nisum.kafka.kafka_training;

import java.time.Duration;
import java.util.Collections;
import java.util.Properties;
import java.util.concurrent.CountDownLatch;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.errors.WakeupException;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConsumerDemoWithThread {
	private Logger logger = LoggerFactory.getLogger(ConsumerDemoWithThread.class.getName());

	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
	        //System.out.println("Hello World");
	        //create consumer configs
		
		new ConsumerDemoWithThread().run();
	       
	}
	private  ConsumerDemoWithThread() {
		
	}
	private void run() {
		
		String bootstrapServers ="localhost:9092";
		String groupId = "my_third_application";
		String topic = "second_topic";
		CountDownLatch latch = new CountDownLatch(1);
		logger.info("creating consumer");
		Runnable myConsumer = new ConsumerThread(latch, bootstrapServers, groupId, topic);
		Thread mythread = new Thread(myConsumer);
		mythread.start();
		
		//add shutdown
		Runtime.getRuntime().addShutdownHook(new Thread(
				() -> {
					logger.info("caught shutdown hook");
					((ConsumerThread) myConsumer).shutdown();
					try {
						latch.await();
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					logger.info("Application exited");
				}
				
				));
		try {
			latch.await();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			logger.error("Application got interrupted",e);
			e.printStackTrace();
		}finally {
			logger.info("Application is closing ");
		}
		
	}

	public class ConsumerThread implements Runnable{
		private Logger logger = LoggerFactory.getLogger(ConsumerThread.class.getName());
		
		private CountDownLatch latch;
		private KafkaConsumer<String,String> consumer;
		
		public ConsumerThread(CountDownLatch latch,String bootstrapServers,String groupId,String topic) {
			this.latch=latch;
			Properties prop = new Properties();
	        prop.setProperty(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG,bootstrapServers);
	        prop.setProperty(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
	        prop.setProperty(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG,StringDeserializer.class.getName());
	        prop.setProperty(ConsumerConfig.GROUP_ID_CONFIG,groupId);
	        prop.setProperty(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG,"earliest");
			consumer=new KafkaConsumer<String,String>(prop);
			  consumer.subscribe(Collections.singleton(topic));
		}
		

		public void run() {
			try {
			   while(true){
			          ConsumerRecords<String,String> reoords= consumer.poll(Duration.ofMillis(100));
			          for(ConsumerRecord record:reoords){
			              logger.info("Key: " + record.key()+"value" +record.value());
			              logger.info("Partition:" + record.partition()+"offset"+record.offset());

			          }
			        }
			}catch(WakeupException we) {
				logger.info("Receivied shutdown singnal");
				
			}finally {
				consumer.close();
				latch.countDown();//tell main method we are done with consumer thread
			}
		}
		public void shutdown() {
			//wakeup menthod is a special mehtod to interrup consumer.poll()
			//it will throw the exception called wakeup
			consumer.wakeup();
			
		}
		
	}

}


